import json

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = r"""\
@article{clark-etal-2020-tydi,
    title = "{T}y{D}i {QA}: A Benchmark for Information-Seeking Question Answering in Typologically Diverse Languages",
    author = "Clark, Jonathan H.  and
      Choi, Eunsol  and
      Collins, Michael  and
      Garrette, Dan  and
      Kwiatkowski, Tom  and
      Nikolaev, Vitaly  and
      Palomaki, Jennimaria",
    editor = "Johnson, Mark  and
      Roark, Brian  and
      Nenkova, Ani",
    journal = "Transactions of the Association for Computational Linguistics",
    volume = "8",
    year = "2020",
    address = "Cambridge, MA",
    publisher = "MIT Press",
    url = "https://aclanthology.org/2020.tacl-1.30",
    doi = "10.1162/tacl_a_00317",
    pages = "454--470",
    abstract = "Confidently making progress on multilingual modeling requires challenging, trustworthy evaluations.
    We present TyDi QA{---}a question answering dataset covering 11 typologically diverse languages with 204K
    question-answer pairs. The languages of TyDi QA are diverse with regard to their typology{---}the set of
    linguistic features each language expresses{---}such that we expect models performing well on this set to
    generalize across a large number of the world{'}s languages. We present a quantitative analysis of the data
    quality and example-level qualitative linguistic analyses of observed language phenomena that would not be found
    in English-only corpora. To provide a realistic information-seeking task and avoid priming effects, questions are
    written by people who want to know the answer, but don{'}t know the answer yet, and the data is collected directly
    in each language without the use of translation.",
}

@inproceedings{cahyawijaya-etal-2021-indonlg,
    title = "{I}ndo{NLG}: Benchmark and Resources for Evaluating {I}ndonesian Natural Language Generation",
    author = "Cahyawijaya, Samuel  and
      Winata, Genta Indra  and
      Wilie, Bryan  and
      Vincentio, Karissa  and
      Li, Xiaohong  and
      Kuncoro, Adhiguna  and
      Ruder, Sebastian  and
      Lim, Zhi Yuan  and
      Bahar, Syafri  and
      Khodra, Masayu  and
      Purwarianti, Ayu  and
      Fung, Pascale",
    booktitle = "Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing",
    month = nov,
    year = "2021",
    address = "Online and Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.emnlp-main.699",
    doi = "10.18653/v1/2021.emnlp-main.699",
    pages = "8875--8898"
}
"""

_DATASETNAME = "tydiqa"

_DESCRIPTION = """\
    TyDi QA is a question answering dataset covering 11 typologically diverse languages with 204K question-answer pairs.
    The languages of TyDi QA are diverse with regard to their typology -- the set of linguistic features that each language
    expresses -- such that we expect models performing well on this set to generalize across a large number of the languages
    in the world. It contains language phenomena that would not be found in English-only corpora. To provide a realistic
    information-seeking task and avoid priming effects, questions are written by people who want to know the answer, but
    don’t know the answer yet, (unlike SQuAD and its descendents) and the data is collected directly in each language
    without the use of translation (unlike MLQA and XQuAD).
    """

_HOMEPAGE = "https://github.com/google-research-datasets/tydiqa"
_LICENSE = Licenses.APACHE_2_0.value
_HF_URL = "https://huggingface.co/datasets/tydiqa"
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_LANGUAGES = ["ind", "tha"]
_LOCAL = False
_SOURCE_VERSION = "1.0.0"
_SOURCE_VERSION_P = "1.0.0"
_SOURCE_VERSION_S = "1.1.0"
_SEACROWD_VERSION = "1.0.0"

_URL = "https://storage.googleapis.com/tydiqa/"
_PRIMARY_URLS = {
    "train": _URL + "v1.0/tydiqa-v1.0-train.jsonl.gz",
    "dev": _URL + "v1.0/tydiqa-v1.0-dev.jsonl.gz",
}
_SECONDARY_URLS = {
    "train": _URL + "v1.1/tydiqa-goldp-v1.1-train.json",
    "dev": _URL + "v1.1/tydiqa-goldp-v1.1-dev.json",
}

_SELECTP_DESP = """Passage selection task (SelectP): Given a list of the passages in the article, return either (a) the index of
          the passage that answers the question or (b) NULL if no such passage exists.
          """
_MINSPAN_DESP = """Minimal answer span task (MinSpan): Given the full text of an article, return one of (a) the start and end
          byte indices of the minimal span that completely answers the question; (b) YES or NO if the question requires
          a yes/no answer and we can draw a conclusion from the passage; (c) NULL if it is not possible to produce a
          minimal answer for this question."""
_GOLDP_DESP = """Gold passage task (GoldP): Given a passage that is guaranteed to contain the
          answer, predict the single contiguous span of characters that answers the question. This is more similar to
          existing reading comprehension datasets (as opposed to the information-seeking task outlined above).
          """
_ID_DESP = """{I}ndo{NLG}: Benchmark and Resources for Evaluating {I}ndonesian Natural Language Generation, is a benchmark
          for evaluating Indonesian natural language generation (NLG) systems. The question-answer pairs are collected
          for each language without using translation services. It uses the Indonesian data from the secondary Gold
          passage task of the TyDiQA dataset.  As the original dataset only provides training and validation sets,
          TydiQA-ID randomly split off 15% of the training data and use it as the test set.
          """


def config_constructor(subset_id, schema, desc, version):
    return SEACrowdConfig(name=f"{_DATASETNAME}_{subset_id}_{schema}", description=desc, version=datasets.Version(version), schema=schema, subset_id=subset_id)


class TydiqaDataset(datasets.GeneratorBasedBuilder):
    """
    This is a main class of SEACrowd dataloader for TyDi QA, which is a question answering dataset covering 11 typologically
    diverse languages with 204K question-answer pairs. The languages of TyDi QA are diverse with regard to their typology.
    Here we also specially provide the split on the primary and secondary task for SEA language like indonesian and thai.
    """

    BUILDER_CONFIGS = [
        # source schema
        # selectp source schema
        config_constructor(subset_id="selectp", schema="source", desc=_SELECTP_DESP, version=_SOURCE_VERSION_P),
        config_constructor(subset_id="selectp_ind", schema="source", desc=_SELECTP_DESP, version=_SOURCE_VERSION_P),
        config_constructor(subset_id="selectp_tha", schema="source", desc=_SELECTP_DESP, version=_SOURCE_VERSION_P),
        # minspan source schema
        config_constructor(subset_id="minspan", schema="source", desc=_MINSPAN_DESP, version=_SOURCE_VERSION_P),
        config_constructor(subset_id="minspan_ind", schema="source", desc=_MINSPAN_DESP, version=_SOURCE_VERSION_P),
        config_constructor(subset_id="minspan_tha", schema="source", desc=_MINSPAN_DESP, version=_SOURCE_VERSION_P),
        # goldp source schema
        config_constructor(subset_id="goldp", schema="source", desc=_GOLDP_DESP, version=_SOURCE_VERSION_S),
        config_constructor(subset_id="goldp_ind", schema="source", desc=_GOLDP_DESP, version=_SOURCE_VERSION_S),
        # tydiqa_id source schema
        config_constructor(subset_id="id", schema="source", desc=_ID_DESP, version=_SOURCE_VERSION_P),
        # seacrowd schema
        # selectp seacrowd schema
        config_constructor(subset_id="selectp", schema="seacrowd_qa", desc=_SELECTP_DESP, version=_SEACROWD_VERSION),
        config_constructor(subset_id="selectp_ind", schema="seacrowd_qa", desc=_SELECTP_DESP, version=_SEACROWD_VERSION),
        config_constructor(subset_id="selectp_tha", schema="seacrowd_qa", desc=_SELECTP_DESP, version=_SEACROWD_VERSION),
        # minspan seacrowd schema
        config_constructor(subset_id="minspan", schema="seacrowd_qa", desc=_MINSPAN_DESP, version=_SEACROWD_VERSION),
        config_constructor(subset_id="minspan_ind", schema="seacrowd_qa", desc=_MINSPAN_DESP, version=_SEACROWD_VERSION),
        config_constructor(subset_id="minspan_tha", schema="seacrowd_qa", desc=_MINSPAN_DESP, version=_SEACROWD_VERSION),
        # goldp seacrowd schema
        config_constructor(subset_id="goldp", schema="seacrowd_qa", desc=_GOLDP_DESP, version=_SEACROWD_VERSION),
        config_constructor(subset_id="goldp_ind", schema="seacrowd_qa", desc=_GOLDP_DESP, version=_SEACROWD_VERSION),
        # tydiqa_id seacrowd schema
        config_constructor(subset_id="id", schema="seacrowd_qa", desc=_ID_DESP, version=_SEACROWD_VERSION),
    ]
    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_id_source"

    def _info(self):
        if ("selectp" in self.config.name) or ("minspan" in self.config.name):
            if "source" in self.config.name:
                features = datasets.Features(
                    {
                        "passage_answer_candidates": datasets.features.Sequence(
                            {
                                "plaintext_start_byte": datasets.Value("int32"),
                                "plaintext_end_byte": datasets.Value("int32"),
                            }
                        ),
                        "question_text": datasets.Value("string"),
                        "document_title": datasets.Value("string"),
                        "language": datasets.Value("string"),
                        "annotations": datasets.features.Sequence(
                            {
                                "passage_answer_candidate_index": datasets.Value("int32"),
                                "minimal_answers_start_byte": datasets.Value("int32"),
                                "minimal_answers_end_byte": datasets.Value("int32"),
                                "yes_no_answer": datasets.Value("string"),
                            }
                        ),
                        "document_plaintext": datasets.Value("string"),
                        "document_url": datasets.Value("string"),
                    }
                )
            elif "seacrowd" in self.config.name:
                features = schemas.qa_features
                features["meta"] = {
                    "passage_answer_candidates": datasets.features.Sequence(
                        {
                            "plaintext_start_byte": datasets.Value("int32"),
                            "plaintext_end_byte": datasets.Value("int32"),
                        }
                    ),
                    "annotations": datasets.features.Sequence(
                        {
                            "passage_answer_candidate_index": datasets.Value("int32"),
                            "minimal_answers_start_byte": datasets.Value("int32"),
                            "minimal_answers_end_byte": datasets.Value("int32"),
                            "yes_no_answer": datasets.Value("string"),
                        }
                    ),
                    "language": datasets.Value("string"),
                }

        elif ("goldp" in self.config.name) or ("tydiqa_id" in self.config.name):
            if "source" in self.config.name:
                features = datasets.Features(
                    {
                        "id": datasets.Value("string"),
                        "title": datasets.Value("string"),
                        "context": datasets.Value("string"),
                        "question": datasets.Value("string"),
                        "answers": datasets.features.Sequence(
                            {
                                "text": datasets.Value("string"),
                                "answer_start": datasets.Value("int32"),
                            }
                        ),
                    }
                )
            elif "seacrowd" in self.config.name:
                features = schemas.qa_features
                features["meta"] = {
                    "answer_start": datasets.Sequence(datasets.Value("int32")),
                }
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            citation=_CITATION,
            homepage=_HOMEPAGE,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        primary_downloaded = dl_manager.download_and_extract(_PRIMARY_URLS)
        secondary_downloaded = dl_manager.download_and_extract(_SECONDARY_URLS)

        if ("selectp" in self.config.name) or ("minspan" in self.config.name):
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"filepath": primary_downloaded["train"]},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"filepath": primary_downloaded["dev"]},
                ),
            ]

        elif "goldp" in self.config.name:
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"filepath": secondary_downloaded["train"]},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"filepath": secondary_downloaded["dev"]},
                ),
            ]
        elif "tydiqa_id" in self.config.name:
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"filepath": secondary_downloaded["train"], "split": "train"},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"filepath": secondary_downloaded["train"], "split": "test"},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"filepath": secondary_downloaded["dev"], "split": "validation"},
                ),
            ]

    def _generate_examples(self, filepath, split=None):
        """Yields examples."""

        if ("selectp" in self.config.name) or ("minspan" in self.config.name):
            with open(filepath, encoding="utf-8") as f:
                for id_, row in enumerate(f):
                    data = json.loads(row)
                    passages = data["passage_answer_candidates"]
                    end_byte = [passage["plaintext_end_byte"] for passage in passages]
                    start_byte = [passage["plaintext_start_byte"] for passage in passages]
                    title = data["document_title"]
                    lang = data["language"]
                    question = data["question_text"]
                    annotations = data["annotations"]
                    yes_no_answers = [annotation["yes_no_answer"] for annotation in annotations]
                    min_answers_end_byte = [annotation["minimal_answer"]["plaintext_end_byte"] for annotation in annotations]
                    min_answers_start_byte = [annotation["minimal_answer"]["plaintext_start_byte"] for annotation in annotations]
                    passage_cand_answers = [annotation["passage_answer"]["candidate_index"] for annotation in annotations]
                    doc = data["document_plaintext"]
                    url = data["document_url"]
                    if (self.config.name == "tydiqa_selectp_source") or (self.config.name == "tydiqa_minspan_source"):
                        yield id_, primary_source_helper(id_, start_byte, end_byte, question, title, lang, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, doc, url)
                    elif (self.config.name == "tydiqa_selectp_ind_source") or (self.config.name == "tydiqa_minspan_ind_source"):
                        if lang == "indonesian":
                            yield id_, primary_source_helper(id_, start_byte, end_byte, question, title, lang, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, doc, url)
                    elif (self.config.name == "tydiqa_selectp_tha_source") or (self.config.name == "tydiqa_minspan_tha_source"):
                        if lang == "thai":
                            yield id_, primary_source_helper(id_, start_byte, end_byte, question, title, lang, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, doc, url)
                    # seacrowd
                    elif (self.config.name == "tydiqa_selectp_seacrowd_qa") or (self.config.name == "tydiqa_minspan_seacrowd_qa"):
                        yield id_, primary_seacrowd_helper(id_, title, question, doc, start_byte, end_byte, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, lang)
                    elif (self.config.name == "tydiqa_selectp_ind_seacrowd_qa") or (self.config.name == "tydiqa_minspan_ind_seacrowd_qa"):
                        if lang == "indonesian":
                            yield id_, primary_seacrowd_helper(id_, title, question, doc, start_byte, end_byte, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, lang)
                    elif (self.config.name == "tydiqa_selectp_tha_seacrowd_qa") or (self.config.name == "tydiqa_minspan_tha_seacrowd_qa"):
                        if lang == "thai":
                            yield id_, primary_seacrowd_helper(id_, title, question, doc, start_byte, end_byte, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, lang)
                    else:
                        raise ValueError(f"No configs to match {self.config.name} in primary_task")

        elif ("goldp" in self.config.name) or ("tydiqa_id" in self.config.name):
            with (open(filepath, encoding="utf-8") as f):
                data = json.load(f)
                tydiqa_id_num = 0
                for article in data["data"]:
                    title = article.get("title", "").strip()
                    for paragraph in article["paragraphs"]:
                        context = paragraph["context"].strip()
                        for qa in paragraph["qas"]:
                            question = qa["question"].strip()
                            id_ = qa["id"]
                            answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                            answers = [answer["text"].strip() for answer in qa["answers"]]
                            if self.config.name == "tydiqa_goldp_source":
                                yield id_, second_source_helper(id_, title, context, question, answer_starts, answers)

                            elif self.config.name == "tydiqa_goldp_ind_source":
                                if id_.startswith("indonesian"):
                                    yield id_, second_source_helper(id_, title, context, question, answer_starts, answers)
                            elif self.config.name == "tydiqa_id_source":
                                if id_.startswith("indonesian"):
                                    tydiqa_id_num += 1
                                    if split == "train" and tydiqa_id_num >= 856:
                                        yield id_, second_source_helper(id_, title, context, question, answer_starts, answers)
                                    if split == "test" and tydiqa_id_num < 856:
                                        yield id_, second_source_helper(id_, title, context, question, answer_starts, answers)
                                    if split == "validation":
                                        yield id_, second_source_helper(id_, title, context, question, answer_starts, answers)

                            elif self.config.name == "tydiqa_goldp_seacrowd_qa":
                                yield id_, second_seacrowd_helper(id_, question, context, answers, answer_starts)
                            elif self.config.name == "tydiqa_goldp_ind_seacrowd_qa":
                                if id_.startswith("indonesian"):
                                    yield id_, second_seacrowd_helper(id_, question, context, answers, answer_starts)
                            elif self.config.name == "tydiqa_id_seacrowd_qa":
                                if id_.startswith("indonesian"):
                                    tydiqa_id_num += 1
                                    if split == "train" and tydiqa_id_num >= 856:
                                        yield id_, second_seacrowd_helper(id_, question, context, answers, answer_starts)
                                    if split == "test" and tydiqa_id_num < 856:
                                        yield id_, second_seacrowd_helper(id_, question, context, answers, answer_starts)
                                    if split == "validation":
                                        yield id_, second_seacrowd_helper(id_, question, context, answers, answer_starts)
                            else:
                                raise ValueError(f"No configs to match {self.config.name} in secondary_task")


def primary_source_helper(id_, start_byte, end_byte, question, title, lang, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, doc, url):
    return {
        "passage_answer_candidates": {
            "plaintext_start_byte": start_byte,
            "plaintext_end_byte": end_byte,
        },
        "question_text": question,
        "document_title": title,
        "language": lang,
        "annotations": {
            "passage_answer_candidate_index": passage_cand_answers,
            "minimal_answers_start_byte": min_answers_start_byte,
            "minimal_answers_end_byte": min_answers_end_byte,
            "yes_no_answer": yes_no_answers,
        },
        "document_plaintext": doc,
        "document_url": url,
    }


def primary_seacrowd_helper(id_, title, question, doc, start_byte, end_byte, passage_cand_answers, min_answers_start_byte, min_answers_end_byte, yes_no_answers, lang):
    return {
        "id": str(id_),
        "question_id": title,
        "document_id": title,
        "question": question,
        "type": "multiple_choice",
        "choices": [""],
        "context": doc,
        "answer": [""],
        "meta": {
            "passage_answer_candidates": {
                "plaintext_start_byte": start_byte,
                "plaintext_end_byte": end_byte,
            },
            "annotations": {
                "passage_answer_candidate_index": passage_cand_answers,
                "minimal_answers_start_byte": min_answers_start_byte,
                "minimal_answers_end_byte": min_answers_end_byte,
                "yes_no_answer": yes_no_answers,
            },
            "language": lang,
        },
    }


def second_source_helper(id_, title, context, question, answer_starts, answers):
    return {
        "title": title,
        "context": context,
        "question": question,
        "id": id_,
        "answers": {
            "answer_start": answer_starts,
            "text": answers,
        },
    }


def second_seacrowd_helper(id_, question, context, answers, answer_starts):
    return {
        "id": id_,
        "question_id": id_,
        "document_id": id_,
        "question": question,
        "type": "abstractive",
        "choices": [],
        "context": context,
        "answer": answers,
        "meta": {"answer_start": answer_starts},
    }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{DBLP:journals/corr/abs-2204-02658,
 author    = {Yingwen Fu and
              Jinyi Chen and
              Nankai Lin and
              Xixuan Huang and
              Xin Ying Qiu and
              Shengyi Jiang},
 title     = {Yunshan Cup 2020: Overview of the Part-of-Speech Tagging Task for
              Low-resourced Languages},
 journal   = {CoRR},
 volume    = {abs/2204.02658},
 year      = {2022},
 url       = {https://doi.org/10.48550/arXiv.2204.02658},
 doi       = {10.48550/arXiv.2204.02658},
 eprinttype = {arXiv},
 eprint    = {2204.02658},
 timestamp = {Tue, 12 Apr 2022 18:42:14 +0200},
 biburl    = {https://dblp.org/rec/journals/corr/abs-2204-02658.bib},
 bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DATASETNAME = "yunshan_cup_2020"

_DESCRIPTION = """\
Lao POS dataset containing 11,000 sentences was released as part of Yunshan-Cup-2020 evaluation track.
"""

_HOMEPAGE = "https://github.com/GKLMIP/Yunshan-Cup-2020"

_LOCAL = False
_LANGUAGES = ["lao"]

_LICENSE = Licenses.UNKNOWN.value  # example: Licenses.MIT.value, Licenses.CC_BY_NC_SA_4_0.value, Licenses.UNLICENSE.value, Licenses.UNKNOWN.value

_URLS = {
    "train": "https://raw.githubusercontent.com/GKLMIP/Yunshan-Cup-2020/main/train.txt",
    "val": "https://raw.githubusercontent.com/GKLMIP/Yunshan-Cup-2020/main/dev.txt",
    "test": "https://raw.githubusercontent.com/GKLMIP/Yunshan-Cup-2020/main/test.txt",
}
_SUPPORTED_TASKS = [Tasks.POS_TAGGING]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class YunshanCup2020Dataset(datasets.GeneratorBasedBuilder):
    """Lao POS dataset containing 11,000 sentences was released as part of Yunshan-Cup-2020 evaluation track."""

    class_labels = ["IAC", "COJ", "ONM", "PRE", "PRS", "V", "DBQ", "IBQ", "FIX", "N", "ADJ", "DMN", "IAQ", "CLF", "PRA", "DAN", "NEG", "NTR", "REL", "PVA", "TTL", "DAQ", "PRN", "ADV", "PUNCT", "CNM"]

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="yunshan_cup_2020 source schema",
            schema="source",
            subset_id="yunshan_cup_2020",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description="yunshan_cup_2020 SEACrowd schema",
            schema="seacrowd_seq_label",
            subset_id="yunshan_cup_2020",
        ),
    ]

    DEFAULT_CONFIG_NAME = "yunshan_cup_2020_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "tokens": [datasets.Value("string")],
                    "pos_tags": [datasets.Value("string")],
                }
            )
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.class_labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        path_dict = dl_manager.download_and_extract(_URLS)
        train_path, val_path, test_path = path_dict["train"], path_dict["val"], path_dict["test"]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": val_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        df = load_postagging_data(filepath)
        if self.config.schema == "source":
            for i, row in enumerate(df):
                ex = {
                    "index": str(i),
                    "tokens": row["sentence"],
                    "pos_tags": row["label"],
                }
                yield i, ex

        elif self.config.schema == "seacrowd_seq_label":
            for i, row in enumerate(df):
                ex = {
                    "id": str(i),
                    "tokens": row["sentence"],
                    "labels": row["label"],
                }
                yield i, ex


def load_postagging_data(file_path):
    data = open(file_path, "r").readlines()
    dataset = []
    sentence, seq_label = [], []
    for line in data:
        if len(line.strip()) > 0:
            token, label = " ", ""
            if len(line.strip().split(" ")) < 2:
                label = line.strip()
            else:
                token, label = line[:-1].split(" ")
            sentence.append(token)
            seq_label.append(label)
        else:
            dataset.append({"sentence": sentence, "label": seq_label})
            sentence = []
            seq_label = []
    return dataset



from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "stif_indonesia"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{wibowo2020semi,
  title={Semi-supervised low-resource style transfer of indonesian informal to formal language with iterative forward-translation},
  author={Wibowo, Haryo Akbarianto and Prawiro, Tatag Aziz and Ihsan, Muhammad and Aji, Alham Fikri and Prasojo, Radityo Eko and Mahendra, Rahmad and Fitriany, Suci},
  booktitle={2020 International Conference on Asian Language Processing (IALP)},
  pages={310--315},
  year={2020},
  organization={IEEE}
}
"""

_DESCRIPTION = """\
STIF-Indonesia is formal-informal (bahasa baku - bahasa alay/slang) style transfer for Indonesian. Texts were collected from Twitter. Then, native speakers were aksed to transform the text into formal style.
"""

_HOMEPAGE = "https://github.com/haryoa/stif-indonesia"

_LICENSE = "MIT"

_BASEURL = "https://raw.githubusercontent.com/haryoa/stif-indonesia/main/data/labelled/"
_URLs = {
    "dev.for": _BASEURL + "dev.for",
    "dev.inf": _BASEURL + "dev.inf",
    "test.for": _BASEURL + "test.for",
    "test.inf": _BASEURL + "test.inf",
    "train.for": _BASEURL + "train.for",
    "train.inf": _BASEURL + "train.inf",
}

_SUPPORTED_TASKS = [Tasks.PARAPHRASING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class STIFIndonesia(datasets.GeneratorBasedBuilder):
    """STIF-Indonesia is formal-informal/colloquial style transfer for Indonesian."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="stif_indonesia_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="STIF Indonesia source schema",
            schema="source",
            subset_id="stif_indonesia",
        ),
        SEACrowdConfig(
            name="stif_indonesia_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="STIF Indonesia Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="stif_indonesia",
        ),
    ]

    DEFAULT_CONFIG_NAME = "stif_indonesia_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "formal": datasets.Value("string"), "informal": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_files = {}
        for key in _URLs:
            data_files[key] = Path(dl_manager.download_and_extract(_URLs[key]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {
                        "formal": data_files["test.for"],
                        "informal": data_files["test.inf"],
                    }
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": {
                        "formal": data_files["dev.for"],
                        "informal": data_files["dev.inf"],
                    }
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "formal": data_files["train.for"],
                        "informal": data_files["train.inf"],
                    }
                },
            ),
        ]

    def _generate_examples(self, filepath: Path):
        data_for = open(filepath["formal"], "r").readlines()
        data_inf = open(filepath["informal"], "r").readlines()

        if self.config.schema == "source":
            for id, (row_for, row_inf) in enumerate(zip(data_for, data_inf)):
                ex = {"id": id, "formal": row_for.strip(), "informal": row_inf.strip()}
                yield id, ex
        elif self.config.schema == "seacrowd_t2t":
            for id, (row_for, row_inf) in enumerate(zip(data_for, data_inf)):
                ex = {
                    "id": id,
                    "text_1": row_for.strip(),
                    "text_2": row_inf.strip(),
                    "text_1_name": "formal",
                    "text_2_name": "informal",
                }
                yield id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import typing as T

from conllu.exceptions import ParseException
from conllu.models import Metadata, TokenList
from conllu.parser import (DEFAULT_FIELD_PARSERS, DEFAULT_FIELDS,
                           _FieldParserType, _MetadataParserType,
                           parse_comment_line, parse_line)

imputed_sent_id: int = 1


def parse_token_and_impute_metadata(data: str, fields: T.Optional[T.Sequence[str]] = None, field_parsers: T.Optional[T.Dict[str, _FieldParserType]] = None, metadata_parsers: T.Optional[T.Dict[str, _MetadataParserType]] = None) -> TokenList:
    """
    Overrides conllu.parse_token_and_metadata via monkey patching.
    This function imputes the following metadata, if these are not found in the .conllu file:
        - sent_id (int): an integer identifier for each sentence.
        - text (str): a concatenated string of token forms. This assumes that all token forms
        are separated with an empty space ' ', and does not consider the `SpaceAfter` field.
    """

    if not data:
        raise ParseException("Can't create TokenList, no data sent to constructor.")

    fields = fields or DEFAULT_FIELDS
    global imputed_sent_id

    if not field_parsers:
        field_parsers = DEFAULT_FIELD_PARSERS.copy()
    elif sorted(field_parsers.keys()) != sorted(fields):
        new_field_parsers = DEFAULT_FIELD_PARSERS.copy()
        new_field_parsers.update(field_parsers)
        field_parsers = new_field_parsers

    tokens = []
    metadata = Metadata()

    for line in data.split('\n'):
        line = line.strip()

        if not line:
            continue

        if line.startswith('#'):
            pairs = parse_comment_line(line, metadata_parsers=metadata_parsers)
            for key, value in pairs:
                metadata[key] = value

        else:
            tokens.append(parse_line(line, fields, field_parsers))

    if 'sent_id' not in metadata:
        metadata['sent_id'] = str(imputed_sent_id)
        imputed_sent_id += 1

    if 'text' not in metadata:
        imputed_text = ""
        for token in tokens:
            imputed_text += str(token['form']) + " "
        metadata['text'] = imputed_text

    return TokenList(tokens, metadata, default_fields=fields)



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@inproceedings{chenchen2017statistical,
    author    = {Ding Chenchen and
                Chea Vichet and
                Pa Win Pa and
                Utiyama Masao and
                Sumita Eiichiro},
    title     = {Statistical Romanization for Abugida Scripts: Data and Experiment on Khmer and Burmese},
    booktitle = {Proceedings of the 23rd Annual Conference of the Association for Natural Language Processing,
                {NLP2017}, Tsukuba, Japan, 13-17 March 2017},
    year      = {2017},
    url       = {https://www.anlp.jp/proceedings/annual_meeting/2017/pdf_dir/P5-7.pdf},
}
"""

_DATASETNAME = "burmese_romanize"
_DESCRIPTION = """\
This dataset consists of 2,335 Burmese names from real university students and faculty,
public figures, and minorities from Myanmar. Each entry includes the original name in
Burmese script, its corresponding Romanization, and the aligned Burmese and Latin
graphemes.
"""

_HOMEPAGE = "http://www.nlpresearch-ucsy.edu.mm/NLP_UCSY/name-db.html"
_LANGUAGES = ["mya"]
_LICENSE = Licenses.CC_BY_NC_SA_4_0.value
_URLS = "http://www.nlpresearch-ucsy.edu.mm/NLP_UCSY/myanmaroma.zip"

_SUPPORTED_TASKS = [Tasks.TRANSLITERATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


class BurmeseRomanizeDataset(datasets.GeneratorBasedBuilder):
    """Romanization of names in Burmese script"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "original": datasets.Value("string"),
                    "romanized": datasets.Value("string"),
                    "aligned_graphemes": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_dir = Path(dl_manager.download_and_extract(_URLS)) / "myanmaroma"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir / "myanmaroma.txt",
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        df = pd.read_csv(filepath, sep=" \|\|\| ", engine='python', header=None, names=["ori", "roman", "seg"])
        if self.config.schema == "source":
            for i, row in df.iterrows():
                yield i, {
                    "original": row["ori"],
                    "romanized": row["roman"],
                    "aligned_graphemes": row["seg"].strip().split(),
                }
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in df.iterrows():
                yield i, {
                    "id": str(i),
                    "text_1": row["ori"],
                    "text_2": row["roman"],
                    "text_1_name": "original",
                    "text_2_name": "romanized",
                }


import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{kautsar2023indotod,
    author={Kautsar, Muhammad Dehan Al and Nurdini, Rahmah Khoirussyifa' and Cahyawijaya, Samuel and Winata, Genta Indra and Purwarianti, Ayu},
    title={IndoToD: A Multi-Domain Indonesian Benchmark For End-to-End Task-Oriented Dialogue Systems},
    journal={arXiv preprint arXiv:2311.00958},
    year={2023},
}
"""

_LANGUAGES = ["ind"]
_LOCAL = False

_DATASETNAME = "indosmd"

_DESCRIPTION = """\
IndoSMD is a synthetic task-oriented dialogue system dataset that was translated from the In-Car Assistant (SMD) dataset (Eric et al., 2017) into the new Indonesian dataset using the translation pipeline method
including delexicalization, translation, and delexicalization. The dataset consists of 323 dialogues in the POI Navigation, Calendar Scheduling, and Weather Information Retrieval domain, with a user and an agent talking to each other.
It also consists of slots and dialogue acts from the user and the agent.
"""

_HOMEPAGE = "https://github.com/dehanalkautsar/IndoToD/tree/main/IndoSMD"

_LICENSE = Licenses.CC_BY_SA_4_0.value

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/dehanalkautsar/IndoToD/main/IndoSMD/IndoSMD_split/IndoSMD_train.json",
        "validation": "https://raw.githubusercontent.com/dehanalkautsar/IndoToD/main/IndoSMD/IndoSMD_split/IndoSMD_dev.json",
        "test": "https://raw.githubusercontent.com/dehanalkautsar/IndoToD/main/IndoSMD/IndoSMD_split/IndoSMD_test.json",
    },
}

_SUPPORTED_TASKS = [Tasks.E2E_TASK_ORIENTED_DIALOGUE]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndoSMDDataset(datasets.GeneratorBasedBuilder):
    """IndoToD: A Multi-Domain Indonesian Benchmark For End-to-End Task-Oriented Dialogue Systems"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="IndoToD: IndoSMD source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_tod",
            version=SEACROWD_VERSION,
            description="IndoToD: IndoSMD SEACrowd End-to-end Task Oriented Dialogue schema",
            schema="seacrowd_tod",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indosmd_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "dialogue": [
                        {
                            "turn": datasets.Value("string"),
                            "data": {
                                "end_dialogue": datasets.Value("string"),
                                "utterance": datasets.Value("string"),
                                "delex_utterance": datasets.Value("string"),
                                "requested": {
                                    "distance": datasets.Value("string"),
                                    "traffic_info": datasets.Value("string"),
                                    "poi_type": datasets.Value("string"),
                                    "address": datasets.Value("string"),
                                    "poi": datasets.Value("string"),
                                    "room": datasets.Value("string"),
                                    "agenda": datasets.Value("string"),
                                    "time": datasets.Value("string"),
                                    "date": datasets.Value("string"),
                                    "party": datasets.Value("string"),
                                    "event": datasets.Value("string"),
                                    "weather_attribute": datasets.Value("string"),
                                    "location": datasets.Value("string"),
                                },
                                "slots": {
                                    "distance": datasets.Value("string"),
                                    "traffic_info": datasets.Value("string"),
                                    "poi_type": datasets.Value("string"),
                                    "address": datasets.Value("string"),
                                    "poi": datasets.Value("string"),
                                    "room": datasets.Value("string"),
                                    "agenda": datasets.Value("string"),
                                    "time": datasets.Value("string"),
                                    "date": datasets.Value("string"),
                                    "party": datasets.Value("string"),
                                    "event": datasets.Value("string"),
                                    "weather_attribute": datasets.Value("string"),
                                    "location": datasets.Value("string"),
                                },
                            },
                        }
                    ],
                    "scenario": {
                        "kb": {
                            "items": [
                                {
                                    "distance": datasets.Value("string"),
                                    "traffic_info": datasets.Value("string"),
                                    "poi_type": datasets.Value("string"),
                                    "address": datasets.Value("string"),
                                    "poi": datasets.Value("string"),
                                    "room": datasets.Value("string"),
                                    "agenda": datasets.Value("string"),
                                    "time": datasets.Value("string"),
                                    "date": datasets.Value("string"),
                                    "party": datasets.Value("string"),
                                    "event": datasets.Value("string"),
                                    "monday": datasets.Value("string"),
                                    "tuesday": datasets.Value("string"),
                                    "wednesday": datasets.Value("string"),
                                    "thursday": datasets.Value("string"),
                                    "friday": datasets.Value("string"),
                                    "saturday": datasets.Value("string"),
                                    "sunday": datasets.Value("string"),
                                    "today": datasets.Value("string"),
                                    "location": datasets.Value("string"),
                                }
                            ],
                            "column_names": [datasets.Value("string")],
                            "kb_title": datasets.Value("string"),
                        },
                        "task": {"intent": datasets.Value("string")},
                        "uuid": datasets.Value("string"),
                    },
                }
            )
        elif self.config.schema == "seacrowd_tod":
            features = schemas.tod_features
        else:
            raise NotImplementedError(f"Schema {self.config.schema} has not been implemented")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                    "split": "validation",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        key_slot_constant = ["distance", "traffic_info", "poi_type", "address", "poi", "room", "agenda", "time", "date", "party", "event", "weather_attribute", "location"]
        key_kb_constant = ["distance", "traffic_info", "poi_type", "address", "poi", "room", "agenda", "time", "date", "party", "event", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday", "today", "location"]

        with open(filepath, "r+") as fw:
            data = json.loads(fw.read())

        if self.config.schema == "source":
            for idx, example in enumerate(data):
                example["index"] = str(idx)
                for i in range(len(example["dialogue"])):
                    if "requested" not in example["dialogue"][i]["data"]:  # the difference between user and system utterance (user and system utterance is divided into each dict in the origin dataset)
                        example["dialogue"][i]["data"]["requested"] = {}
                        example["dialogue"][i]["data"]["slots"] = {}
                        for key in key_slot_constant:
                            example["dialogue"][i]["data"]["requested"][key] = ""
                            example["dialogue"][i]["data"]["slots"][key] = ""
                    else:
                        for key in key_slot_constant:
                            if key not in example["dialogue"][i]["data"]["requested"]:
                                example["dialogue"][i]["data"]["requested"][key] = ""
                            if key not in example["dialogue"][i]["data"]["slots"]:
                                example["dialogue"][i]["data"]["slots"][key] = ""

                if not example["scenario"]["kb"].get("items"):
                    example["scenario"]["kb"]["items"] = []

                for i in range(len(example["scenario"]["kb"]["items"])):
                    for key in key_kb_constant:
                        if key not in example["scenario"]["kb"]["items"][i]:
                            example["scenario"]["kb"]["items"][i][key] = ""

                yield str(idx), example

        elif self.config.schema == "seacrowd_tod":
            for idx, tod_dialogue in enumerate(data):
                example = {}
                example["dialogue_idx"] = idx

                dialogue = []
                # NOTE: the dialogue always started with `driver` as first utterance
                for turn, i in enumerate(range(0, len(tod_dialogue["dialogue"]) + 2, 2)):
                    dial = {}
                    dial["turn_idx"] = turn

                    # system_utterance properties
                    dial["system_utterance"] = ""
                    dial["system_acts"] = []
                    if turn != 0:
                        dial["system_utterance"] = tod_dialogue["dialogue"][i - 1]["data"]["utterance"]
                    if i < len(tod_dialogue["dialogue"]):
                        # NOTE: system_acts will be filled with every slot that has 'True' value on the origin dataset (on the requested field)
                        for act in tod_dialogue["dialogue"][i + 1]["data"]["requested"]:
                            if tod_dialogue["dialogue"][i + 1]["data"]["requested"][act]:
                                dial["system_acts"].append([act])

                    # user_utterance properties
                    dial["turn_label"] = []
                    dial["belief_state"] = []
                    if i == len(tod_dialogue["dialogue"]):
                        # case if turn_idx > len(dialogue) --> add dummy user_utterance
                        dial["user_utterance"] = ""
                    else:
                        dial["user_utterance"] = tod_dialogue["dialogue"][i]["data"]["utterance"]
                        # NOTE: belief_state will be filled with request act from `requested` field & inform act from `slots` field in the origin dataset
                        for act in tod_dialogue["dialogue"][i + 1]["data"]["requested"]:
                            if tod_dialogue["dialogue"][i + 1]["data"]["requested"][act]:
                                dial["belief_state"].append({"slots": [["slot", act]], "act": "request"})
                        for slot, slot_value in tod_dialogue["dialogue"][i + 1]["data"]["slots"].items():
                            dial["belief_state"].append({"slots": [[slot, slot_value]], "act": "inform"})

                    # append to dialogue
                    dialogue.append(dial)
                example["dialogue"] = dialogue
                yield str(idx), example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

import pandas as pd

_CITATION = """\
@misc{
   research, 
   title={indonesian-nlp/librivox-indonesia · datasets at hugging face}, 
   url={https://huggingface.co/datasets/indonesian-nlp/librivox-indonesia},
   author={Indonesian-nlp}
} 
"""

_DATASETNAME = "librivox_indonesia"
_DESCRIPTION = """\
The LibriVox Indonesia dataset consists of MP3 audio and a corresponding text file we generated from the public domain audiobooks LibriVox. 
We collected only languages in Indonesia for this dataset. 
The original LibriVox audiobooks or sound files' duration varies from a few minutes to a few hours. 
Each audio file in the speech dataset now lasts from a few seconds to a maximum of 20 seconds.
We converted the audiobooks to speech datasets using the forced alignment software we developed. 
It supports multilingual, including low-resource languages, such as Acehnese, Balinese, or Minangkabau. 
We can also use it for other languages without additional work to train the model.
The dataset currently consists of 8 hours in 7 languages from Indonesia. 
We will add more languages or audio files as we collect them. 
"""

_HOMEPAGE = "https://huggingface.co/indonesian-nlp/librivox-indonesia"

_LICENSE = "CC0"

_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/indonesian-nlp/librivox-indonesia/resolve/main/data",
}
_LANGUAGES = {"ind", "sun", "jav", "min", "bug", "ban", "ace"}
_LANG_CODE = {
    "ind": ["ind", "indonesian"],
    "sun": ["sun", "sundanese"],
    "jav": ["jav", "javanese"],
    "min": ["min", "minangkabau"],
    "bug": ["bug", "bugisnese"],
    "ban": ["bal", "balinese"],
    "ace": ["ace", "acehnese"]
}
_LOCAL = False
_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class LibrivoxIndonesia(datasets.GeneratorBasedBuilder):
    """
    Librivox-indonesia is a speech-to-text dataset in 7 languages available in Indonesia.
    The default dataloader contains all languages, while the other available dataloaders contain a designated language.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="librivox_indonesia_source",
            version=_SOURCE_VERSION,
            description="Librivox-Indonesia source schema for all languages",
            schema="source",
            subset_id="librivox_indonesia",
        )] + [
        SEACrowdConfig(
            name="librivox_indonesia_{lang}_source".format(lang=lang),
            version=_SOURCE_VERSION,
            description="Librivox-Indonesia source schema for {lang} languages".format(lang=_LANG_CODE[lang][1]),
            schema="source",
            subset_id="librivox_indonesia_{lang}".format(lang=lang),
        ) for lang in _LANGUAGES] + [
        SEACrowdConfig(
            name="librivox_indonesia_seacrowd_sptext",
            version=_SEACROWD_VERSION,
            description="Librivox-Indonesia Nusantara schema for all languages",
            schema="seacrowd_sptext",
            subset_id="librivox_indonesia",
        )] + [
        SEACrowdConfig(
            name="librivox_indonesia_{lang}_seacrowd_sptext".format(lang=lang),
            version=_SEACROWD_VERSION,
            description="Librivox-Indonesia Nusantara schema for {lang} languages".format(lang=_LANG_CODE[lang][1]),
            schema="seacrowd_sptext",
            subset_id="librivox_indonesia_{lang}".format(lang=lang),
        )for lang in _LANGUAGES]

    DEFAULT_CONFIG_NAME = "librivox_indonesia_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "path": datasets.Value("string"),
                    "language": datasets.Value("string"),
                    "reader": datasets.Value("string"),
                    "sentence": datasets.Value("string"),
                    "audio": datasets.features.Audio(sampling_rate=44100)
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]

        audio_path = {}
        local_extracted_archive = {}
        metadata_path = {}
        splits = ["train", "test"]
        for split in splits:
            audio_path[split] = dl_manager.download(os.path.join(urls, "audio_{split}.tgz".format(split=split)))
            local_extracted_archive[split] = dl_manager.extract(audio_path[split]) if not dl_manager.is_streaming else None
            metadata_path[split] = dl_manager.download_and_extract(
                os.path.join(urls, "metadata_{split}.csv.gz".format(split=split))
            )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "local_extracted_archive": local_extracted_archive["train"],
                    "audio_path": dl_manager.iter_archive(audio_path["train"]),
                    "metadata_path": metadata_path["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "local_extracted_archive": local_extracted_archive["test"],
                    "audio_path": dl_manager.iter_archive(audio_path["test"]),
                    "metadata_path": metadata_path["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, local_extracted_archive: Path, audio_path, metadata_path: Path, split: str) -> Tuple[int, Dict]:
        df = pd.read_csv(
            metadata_path,
            encoding="utf-8"
        )
        lang = self.config.subset_id.split("_")[-1]
        if lang != "indonesia":
            lang = _LANG_CODE[lang][0]
        path_to_audio = "librivox-indonesia"
        metadata = {}
        for id, row in df.iterrows():
            if lang == row["language"] or lang == "indonesia":
                path = os.path.join(path_to_audio, row["path"])
                metadata[path] = row
                metadata[path]["id"] = id

        for path, f in audio_path:
            if path in metadata:
                row = metadata[path]
                path = os.path.join(local_extracted_archive, path) if local_extracted_archive else path
                if self.config.schema == "source":
                    yield row["id"], {
                        "path": path,
                        "language": row["language"],
                        "reader": row["reader"],
                        "sentence": row["sentence"],
                        "audio": path,
                    }
                elif self.config.schema == "seacrowd_sptext":
                    yield row["id"], {
                        "id": row["id"],
                        "speaker_id": row["reader"],
                        "path": path,
                        "audio": path,
                        "text": row["sentence"],
                        "metadata": {
                            "speaker_age": None,
                            "speaker_gender": None,
                        }
                    }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from cgitb import text
from itertools import chain
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{sakti-icslp-2004,
    title = "Indonesian Speech Recognition for Hearing and Speaking Impaired People",
    author = "Sakti, Sakriani and Hutagaol, Paulus and Arman, Arry Akhmad and Nakamura, Satoshi",
    booktitle = "Proc. International Conference on Spoken Language Processing (INTERSPEECH - ICSLP)",
    year = "2004",
    pages = "1037--1040"
    address = "Jeju Island, Korea"
}
"""
_DATASETNAME = "indspeech_digit_cdsr"
_LANGUAGES = ["ind"]
_DESCRIPTION = """\
INDspeech_DIGIT_CDSR is the first Indonesian speech dataset for connected digit speech recognition (CDSR). The data was developed by TELKOMRisTI (R&D Division, PT Telekomunikasi Indonesia) in collaboration with Advanced Telecommunication Research Institute International (ATR) Japan and Bandung Institute of Technology (ITB) under the Asia-Pacific Telecommunity (APT) project in 2004 [Sakti et al., 2004]. Although it was originally developed for a telecommunication system for hearing and speaking impaired people, it can be used for other applications, i.e., automatic call centers that recognize telephone numbers.
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_digit_cdsr"
_LOCAL = False
_LICENSE = "CC-BY-NC-SA-4.0"

_TMP_URL = {
    "lst": "https://raw.githubusercontent.com/s-sakti/data_indsp_digit_cdsr/main/lst/",
    "text": "https://github.com/s-sakti/data_indsp_digit_cdsr/raw/main/text/",
    "speech": "https://github.com/s-sakti/data_indsp_digit_cdsr/raw/main/speech/",
}

_URLS = {
    "lst": {
        "train_spk": _TMP_URL["lst"] + "train_spk.lst",
        "train_fname": _TMP_URL["lst"] + "train_fname.lst",
        "test_spk": [_TMP_URL["lst"] + "test" + str(i) + "_spk.lst" for i in range(1, 5)],
        "test_fname": [_TMP_URL["lst"] + "test" + str(i) + "_fname.lst" for i in range(1, 5)],
    },
    "train": {"speech": _TMP_URL["speech"] + "train/", "text": _TMP_URL["text"] + "train/"},
    "test": {"speech": _TMP_URL["speech"] + "test", "text": _TMP_URL["text"] + "test"},
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class INDspeechDIGITCDSR(datasets.GeneratorBasedBuilder):
    """Indonesian speech dataset for connected digit speech recognition"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indspeech_digit_cdsr_source",
            version=SOURCE_VERSION,
            description="indspeech_digit_cdsr source schema",
            schema="source",
            subset_id="indspeech_digit_cdsr",
        ),
        SEACrowdConfig(
            name="indspeech_digit_cdsr_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description="indspeech_digit_cdsr Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="indspeech_digit_cdsr",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indspeech_digit_cdsr_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        lst_train_spk = Path(dl_manager.download_and_extract(_URLS["lst"]["train_spk"]))
        lst_train_fname = Path(dl_manager.download_and_extract(_URLS["lst"]["train_fname"]))
        lst_test_spk = [Path(dl_manager.download_and_extract(url)) for url in _URLS["lst"]["test_spk"]]
        lst_test_fname = [Path(dl_manager.download_and_extract(url)) for url in _URLS["lst"]["test_fname"]]

        fnames = {"test": []}
        speech = {"test": {}}
        text = {"test": {}}

        with open(lst_train_spk, "r") as f:
            speakers = [spk.replace("\n", "") for spk in f.readlines()]
            tmp_speech = [Path(dl_manager.download_and_extract(_URLS["train"]["speech"] + spk + ".zip")) for spk in speakers]
            tmp_text = [Path(dl_manager.download_and_extract(_URLS["train"]["text"] + spk + ".zip")) for spk in speakers]
            speech["train"] = {speech[:-4]: os.path.join(spk, speech) for spk in tmp_speech for speech in os.listdir(spk)}
            text["train"] = {text[:-4]: os.path.join(spk, text) for spk in tmp_text for text in os.listdir(spk)}
        f.close()

        with open(lst_train_fname, "r") as f:
            fnames["train"] = [fname.replace("\n", "") for fname in f.readlines()]
        f.close()

        for i in range(1, 5):
            with open(lst_test_fname[i - 1], "r") as f:
                fnames["test"].append([spk.replace("\n", "") for spk in f.readlines()])
            f.close()

            with open(lst_test_spk[i - 1], "r") as f:
                speakers = [spk.replace("\n", "") for spk in f.readlines()]
                tmp_speech = [Path(dl_manager.download_and_extract(_URLS["test"]["speech"] + str(i) + "/" + spk + ".zip")) for spk in speakers]
                tmp_text = [Path(dl_manager.download_and_extract(_URLS["test"]["text"] + str(i) + "/" + spk + ".zip")) for spk in speakers]
                tmp_dict_speech = {speech[:-4]: os.path.join(spk, speech) for spk in tmp_speech for speech in os.listdir(spk)}
                tmp_dict_text = {text[:-4]: os.path.join(spk, text) for spk in tmp_text for text in os.listdir(spk)}
            f.close()

            for k, v in tmp_dict_speech.items():
                if k in speech["test"]:
                    continue
                else:
                    speech["test"][k] = v

            for k, v in tmp_dict_text.items():
                if k in text["test"]:
                    continue
                else:
                    text["test"][k] = v

        fnames["test"] = list(chain(*fnames["test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "fnames": fnames["train"],
                        "speech": speech["train"],
                        "text": text["train"],
                    },
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {
                        "fnames": fnames["test"],
                        "speech": speech["test"],
                        "text": text["test"],
                    },
                    "split": "test",
                },
            ),
        ]

    @staticmethod
    def text_process(utterance_path):
        with open(utterance_path, "r") as f:
            w = [r.replace("\n", "") for r in f.readlines()]
        f.close()
        return " ".join(w[1:-1])

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        had_used = []
        for key, example in enumerate(filepath["fnames"]):
            if example not in had_used:
                had_used.append(example)
                spk_id, _ = example.split("_")
                if self.config.schema == "source":
                    yield key, {
                        "id": example,
                        "speaker_id": spk_id,
                        "gender": spk_id[0],
                        "path": filepath["speech"][example],
                        "audio": filepath["speech"][example],
                        "text": self.text_process(filepath["text"][example]),
                    }

                elif self.config.schema == "seacrowd_sptext":
                    yield key, {
                        "id": example,
                        "speaker_id": spk_id,
                        "text": self.text_process(filepath["text"][example]),
                        "path": filepath["speech"][example],
                        "audio": filepath["speech"][example],
                        "metadata": {
                            "speaker_age": None,
                            "speaker_gender": spk_id[0],
                        },
                    }
            else:
                continue



DOMAIN_LABELS = [
    "messaging",
    "calling",
    "event",
    "timer",
    "music",
    "weather",
    "alarm",
    "people",
    "reminder",
    "recipes",
    "news",
]

INTENT_LABELS = [
    "SEND_MESSAGE",
    "GET_MESSAGE",
    "GET_MESSAGE_CONTACT",
    "CREATE_CALL",
    "GET_CONTACT",
    "GET_AVAILABILITY",
    "SET_RSVP_INTERESTED",
    "SET_RSVP_YES",
    "SET_DEFAULT_PROVIDER_CALLING",
    "PAUSE_TIMER",
    "GET_SUNSET",
    "GET_TIMER",
    "SUBTRACT_TIME_TIMER",
    "RESUME_TIMER",
    "CREATE_ALARM",
    "CREATE_TIMER",
    "PLAY_MUSIC",
    "SET_DEFAULT_PROVIDER_MUSIC",
    "GET_STORIES_NEWS",
    "UPDATE_TIMER",
    "SHARE_EVENT",
    "PLAY_MEDIA",
    "UPDATE_CALL",
    "STOP_SHUFFLE_MUSIC",
    "GET_ALARM",
    "GET_WEATHER",
    "PREFER",
    "GET_LOCATION",
    "GET_DATE_TIME_EVENT",
    "GET_EVENT",
    "GET_CATEGORY_EVENT",
    "UPDATE_METHOD_CALL",
    "END_CALL",
    "SET_UNAVAILABLE",
    "GET_CALL_TIME",
    "GET_CALL",
    "SNOOZE_ALARM",
    "SILENCE_ALARM",
    "DELETE_ALARM",
    "UNLOOP_MUSIC",
    "UPDATE_ALARM",
    "CREATE_PLAYLIST_MUSIC",
    "DELETE_REMINDER",
    "CREATE_REMINDER",
    "ANSWER_CALL",
    "SET_AVAILABLE",
    "SWITCH_CALL",
    "UPDATE_REMINDER_DATE_TIME",
    "GET_REMINDER",
    "GET_REMINDER_DATE_TIME",
    "UPDATE_REMINDER_TODO",
    "IGNORE_CALL",
    "GET_INFO_CONTACT",
    "GET_EMPLOYER",
    "GET_EMPLOYMENT_TIME",
    "GET_EDUCATION_TIME",
    "GET_UNDERGRAD",
    "GET_AGE",
    "GET_EDUCATION_DEGREE",
    "GET_INFO_RECIPES",
    "GET_RECIPES",
    "IS_TRUE_RECIPES",
    "PREVIOUS_TRACK_MUSIC",
    "QUESTION_NEWS",
    "GET_DETAILS_NEWS",
    "ADD_TIME_TIMER",
    "LIKE_MUSIC",
    "ADD_TO_PLAYLIST_MUSIC",
    "QUESTION_MUSIC",
    "SKIP_TRACK_MUSIC",
    "LOOP_MUSIC",
    "PAUSE_MUSIC",
    "GET_TRACK_INFO_MUSIC",
    "DISLIKE_MUSIC",
    "RESTART_TIMER",
    "DELETE_TIMER",
    "DELETE_PLAYLIST_MUSIC",
    "REMOVE_FROM_PLAYLIST_MUSIC",
    "REWIND_MUSIC",
    "REPLAY_MUSIC",
    "START_SHUFFLE_MUSIC",
    "STOP_MUSIC",
    "FAST_FORWARD_MUSIC",
    "FOLLOW_MUSIC",
    "SET_RSVP_NO",
    "GET_ATTENDEE_EVENT",
    "GET_SUNRISE",
    "HOLD_CALL",
    "GET_CALL_CONTACT",
    "GET_REMINDER_AMOUNT",
    "HELP_REMINDER",
    "GET_REMINDER_LOCATION",
    "UPDATE_REMINDER",
    "UPDATE_REMINDER_LOCATION",
    "REPEAT_ALL_OFF_MUSIC",
    "GET_CONTACT_METHOD",
    "GET_JOB",
    "GET_LIFE_EVENT",
    "GET_GENDER",
    "GET_MAJOR",
    "GET_MUTUAL_FRIENDS",
    "GET_LYRICS_MUSIC",
    "CANCEL_MESSAGE",
    "RESUME_CALL",
    "REPEAT_ALL_MUSIC",
    "RESUME_MUSIC",
    "GET_LIFE_EVENT_TIME",
    "GET_LANGUAGE",
    "MERGE_CALL",
    "GET_AIRQUALITY",
]


from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "smsa"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_CITATION = """\
@INPROCEEDINGS{8904199,
    author={Purwarianti, Ayu and Crisdayanti, Ida Ayu Putu Ari},
    booktitle={2019 International Conference of Advanced Informatics: Concepts, Theory and Applications (ICAICTA)},
    title={Improving Bi-LSTM Performance for Indonesian Sentiment Analysis Using Paragraph Vector},
    year={2019},
    pages={1-5},
    doi={10.1109/ICAICTA.2019.8904199}
}

@inproceedings{wilie2020indonlu,
  title={IndoNLU: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
  author={Wilie, Bryan and Vincentio, Karissa and Winata, Genta Indra and Cahyawijaya, Samuel and Li, Xiaohong and Lim, Zhi Yuan and Soleman, Sidik and Mahendra, Rahmad and Fung, Pascale and Bahar, Syafri and others},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  pages={843--857},
  year={2020}
}
"""

_DESCRIPTION = """\
SmSA is a sentence-level sentiment analysis dataset (Purwarianti and Crisdayanti, 2019) is a collection of comments and reviews
in Indonesian obtained from multiple online platforms. The text was crawled and then annotated by several Indonesian linguists
to construct this dataset. There are three possible sentiments on the SmSA dataset: positive, negative, and neutral
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {
    "train": "https://github.com/IndoNLP/indonlu/raw/master/dataset/smsa_doc-sentiment-prosa/train_preprocess.tsv",
    "validation": "https://github.com/IndoNLP/indonlu/raw/master/dataset/smsa_doc-sentiment-prosa/valid_preprocess.tsv",
    "test": "https://github.com/IndoNLP/indonlu/raw/master/dataset/smsa_doc-sentiment-prosa/test_preprocess.tsv",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class SMSA(datasets.GeneratorBasedBuilder):
    """SMSA is a sentiment analysis dataset consisting of 3 labels (positive, neutral, and negative) which comes from comments and reviews collected from multiple online platforms."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="smsa_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="SMSA source schema",
            schema="source",
            subset_id="smsa",
        ),
        SEACrowdConfig(
            name="smsa_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="SMSA Nusantara schema",
            schema="seacrowd_text",
            subset_id="smsa",
        ),
    ]

    DEFAULT_CONFIG_NAME = "smsa_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "sentence": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["negative", "neutral", "positive"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_tsv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_tsv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_tsv_path,
            "validation": validation_tsv_path,
            "test": test_tsv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        df = pd.read_csv(filepath, sep="\t", header=None).reset_index()
        df.columns = ["id", "sentence", "label"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"index": str(row.id), "sentence": row.sentence, "label": row.label}
                yield row.id, ex
        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.sentence,
                    "label": row.label
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@misc{
   research, 
   title={Jakartaresearch/google-play-review · datasets at hugging face}, 
   url={https://huggingface.co/datasets/jakartaresearch/google-play-review},
   author={Research, Jakarta AI}
} 
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "id_google_play_review"
_DESCRIPTION = """\
Indonesian Google Play Review, dataset scrapped from e-commerce app on Google Play for sentiment analysis.
Total number of data: 10041 (train: 7028, validation: 3012). Provided by Jakarta AI Research.
"""

_HOMEPAGE = "https://github.com/jakartaresearch/hf-datasets/tree/main/google-play-review/google-play-review"
_LICENSE = "CC-BY 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://media.githubusercontent.com/media/jakartaresearch/hf-datasets/main/google-play-review/google-play-review/train.csv",
        "valid": "https://media.githubusercontent.com/media/jakartaresearch/hf-datasets/main/google-play-review/google-play-review/validation.csv",
    }
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IDGooglePlayReview(datasets.GeneratorBasedBuilder):
    """
    Indonesian Google Play Review is a dataset containing reviews from Google Play Indonesia, used for sentiment
    analysis.
    The language content is mainly Indonesian, however beware of context-switching (some sentences are partly or
    entirely in English).
    The available labels:
        label: ['pos', 'neg'] for source and seacrowd_text scheme
        stars: [1, 2, 3, 4, 5] for source
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_google_play_review_source",
            version=SOURCE_VERSION,
            description="id_google_play_review source schema",
            schema="source",
            subset_id="id_google_play_review",
        ),
        SEACrowdConfig(
            name="id_google_play_review_posneg_source",
            version=SOURCE_VERSION,
            description="id_google_play_review source schema",
            schema="source",
            subset_id="id_google_play_review_posneg",
        ),
        SEACrowdConfig(
            name="id_google_play_review_seacrowd_text",
            version=SEACROWD_VERSION,
            description="id_google_play_review Nusantara schema, 1-5 stars rating only (for pos/neg labels, please use the subset_id \"id_google_play_review_posneg\")",
            schema="seacrowd_text",
            subset_id="id_google_play_review",
        ),
        SEACrowdConfig(
            name="id_google_play_review_posneg_seacrowd_text",
            version=SEACROWD_VERSION,
            description="id_google_play_review Nusantara schema, pos/neg label only",
            schema="seacrowd_text",
            subset_id="id_google_play_review_posneg",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_google_play_review_source"

    def _info(self) -> datasets.DatasetInfo:

        # Create the source schema; this schema will keep all keys/information/labels as close to the original dataset
        # as possible.

        # You can arbitrarily nest lists and dictionaries.
        # For iterables, use lists over tuples or `datasets.Sequence`

        if self.config.schema == "source":
            features = datasets.Features({
                "text": datasets.Value("string"),
                "label": datasets.Value("string"),
                "stars": datasets.Value("int8")
            })
        elif self.config.schema == "seacrowd_text":
            if self.config.subset_id == "id_google_play_review_posneg":
                features = schemas.text_features(["pos", "neg"])
            elif self.config.subset_id == "id_google_play_review":
                features = schemas.text_features(["1", "2", "3", "4", "5"])
            else:
                raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        train_data_path = Path(dl_manager.download(urls["train"]))
        valid_data_path = Path(dl_manager.download(urls["valid"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_data_path, "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": valid_data_path, "split": "valid"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        df = pd.read_csv(filepath, sep=",").reset_index()
        for row in df.itertuples(index=False):
            if self.config.schema == "source":
                example = {"text": row.text, "label": row.label, "stars": row.stars}
                yield row.index, example
            elif self.config.schema == "seacrowd_text":
                if self.config.subset_id == "id_google_play_review_posneg":
                    example = {"id": row.index, "text": row.text, "label": row.label}
                elif self.config.subset_id == "id_google_play_review":
                    example = {"id": row.index, "text": row.text, "label": str(row.stars)}
                else:
                    raise ValueError(f"Invalid config: {self.config.name}")
                yield row.index, example



import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{riegoenhancement,
  title={Enhancement to Low-Resource Text Classification via Sequential Transfer Learning},
  author={Riego, Neil Christian R. and Villarba, Danny Bell and Sison, Ariel Antwaun Rolando C. and Pineda, Fernandez C. and Lagunzad, Herminiño C.}
  journal={United International Journal for Research & Technology},
  volume={04},
  issue={08},
  pages={72--82}
}
"""

_LOCAL = False
_LANGUAGES = ["fil", "tgl"]
_DATASETNAME = "shopee_reviews_tagalog"
_DESCRIPTION = """\
The Shopee reviews dataset is constructed by randomly taking 2100 training
samples and 450 samples for testing and validation for each review star from 1
to 5. In total, there are 10500 training samples and 2250 each in validation and
testing samples.
"""

_HOMEPAGE = "https://huggingface.co/datasets/scaredmeow/shopee-reviews-tl-stars"
_LICENSE = Licenses.MPL_2_0.value
_URLS = {
    "train": "https://huggingface.co/datasets/scaredmeow/shopee-reviews-tl-stars/resolve/main/train.csv",
    "validation": "https://huggingface.co/datasets/scaredmeow/shopee-reviews-tl-stars/resolve/main/validation.csv",
    "test": "https://huggingface.co/datasets/scaredmeow/shopee-reviews-tl-stars/resolve/main/test.csv",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ShopeeReviewsTagalogDataset(datasets.GeneratorBasedBuilder):
    """Shopee Reviews Tagalog dataset from https://huggingface.co/datasets/scaredmeow/shopee-reviews-tl-stars"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"
    # "N" means N+1 star(s) review, e.g. "2" means 3 stars review
    LABEL_CLASSES = ["0", "1", "2", "3", "4"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        # The SEACrowd schema and the source schema is the same
        features = schemas.text_features(self.LABEL_CLASSES)
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        data_files = {
            "train": Path(dl_manager.download_and_extract(_URLS["train"])),
            "validation": Path(dl_manager.download_and_extract(_URLS["validation"])),
            "test": Path(dl_manager.download_and_extract(_URLS["test"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"], "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"], "split": "validation"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"], "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yield (idx, example) tuples"""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file,
                quotechar='"',
                delimiter=",",
                quoting=csv.QUOTE_ALL,
                skipinitialspace=True,
            )

            next(csv_reader)
            for idx, row in enumerate(csv_reader):
                text, label = row
                example = {"id": idx, "text": text, "label": label}
                yield idx, example
